@@ -20,15 +20,15 @@
 """Holds platform-specific settings.
 """
 
-__id__        = "$Id: platform.py.in 4221 2008-09-15 08:11:23Z wwalker $"
-__version__   = "$Revision: 4221 $"
-__date__      = "$Date: 2008-09-15 04:11:23 -0400 (Mon, 15 Sep 2008) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 # $ORCA_MAJOR_VERSION.$ORCA_MINOR_VERSION.$ORCA_MICRO_VERSION
 #
-version     = "2.24.4"
+version     = "2.28.3"
 
 # "--prefix" parameter used when configuring the build.
 #
@@ -41,3 +41,7 @@ package     = "orca"
 # The name of the data directory (usually "share").
 #
 datadirname = "share"
+
+# The directory where we could find liblouis translation tables.
+#
+tablesdir = ""
