@@ -20,10 +20,10 @@
 """ Custom script for Thunderbird 3.
 """
 
-__id__        = "$Id: script.py 4338 2008-11-17 22:05:37Z joanied $"
-__version__   = "$Revision: 4338 $"
-__date__      = "$Date: 2008-11-17 17:05:37 -0500 (Mon, 17 Nov 2008) $"
-__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import gtk
@@ -163,6 +163,16 @@ class Script(Gecko.Script):
         - event: the Event
         """
 
+        # Much of the Gecko code is designed to handle Gecko's broken
+        # caret navigation. This is not needed in -- and can sometimes
+        # interfere with our presentation of -- a simple message being
+        # composed by the user. Surely we can count on Thunderbird to
+        # handle navigation in that case.
+        # 
+        if self.isEditableMessage(event.source):
+            self.setCaretContext(event.source, event.detail1)
+            return default.Script.onCaretMoved(self, event)
+
         # Page_Up/Page_Down are not used by Orca. However, users report
         # using these keys in Thunderbird without success. The default
         # script is sometimes rejecting the resulting caret-moved events
@@ -184,7 +194,8 @@ class Script(Gecko.Script):
                 event.source.getState().contains(pyatspi.STATE_EDITABLE)
 
         if updatePosition:
-            orca.setLocusOfFocus(event, event.source, False)
+            orca.setLocusOfFocus(
+                event, event.source, notifyPresentationManager=False)
             self.setCaretContext(event.source, event.detail1)
 
             # The Gecko script, should it be about to pass along this
@@ -238,8 +249,15 @@ class Script(Gecko.Script):
             row = table.getRowAtIndex(self.getCellIndex(obj))
             for i in range(0, table.nColumns):
                 acc = table.getAccessibleAt(row, i)
-                if acc.name:                
-                    orca.setLocusOfFocus(event, acc)
+                if acc.name:
+                    # For some reason orca.py's check to see if the
+                    # object we're setting the locusOfFocus to is the
+                    # same as the current locusOfFocus is returning
+                    # True when it's not actually True. Therefore,
+                    # we'll force the propagation as a precaution.
+                    #
+                    force = event.type.startswith("focus:")
+                    orca.setLocusOfFocus(event, acc, force=force)
                     consume = True
                     break
 
@@ -345,12 +363,10 @@ class Script(Gecko.Script):
         # number as the previously deleted message).
         # See bug #536451 for more details.
         #
-        rolesList = [pyatspi.ROLE_TABLE_CELL, \
-                     pyatspi.ROLE_TREE_TABLE, \
-                     pyatspi.ROLE_SCROLL_PANE, \
-                     pyatspi.ROLE_SCROLL_PANE, \
-                     pyatspi.ROLE_FRAME, \
-                     pyatspi.ROLE_APPLICATION]
+        rolesList = [pyatspi.ROLE_TABLE_CELL,
+                     pyatspi.ROLE_TREE_TABLE,
+                     pyatspi.ROLE_SCROLL_PANE,
+                     pyatspi.ROLE_SCROLL_PANE]
         if self.isDesiredFocusedItem(event.source, rolesList):
             if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
                 string = orca_state.lastNonModifierKeyEvent.event_string
@@ -540,7 +556,7 @@ class Script(Gecko.Script):
                 #
                 text = _("%s panel") % parent.name
                 utterances.append(text)
-                speech.speakUtterances(utterances)
+                speech.speak(utterances)
         else:
             grandparent = parent.parent
             if grandparent \
@@ -557,7 +573,7 @@ class Script(Gecko.Script):
                     #
                     text = _("%s panel") % grandparent.name
                     utterances.append(text)
-                    speech.speakUtterances(utterances)
+                    speech.speak(utterances)
 
     def _presentMessage(self, documentFrame):
         """Presents the first line of the message, or the entire message,
@@ -571,11 +587,30 @@ class Script(Gecko.Script):
             self.sayAll(None)
         self._messageLoaded = False
 
+    def getBottomOfFile(self):
+        """Returns the object and last caret offset at the bottom of the
+        document frame. Overridden here to handle editable messages.
+        """
+
+        # Pylint thinks that obj is an instance of a list. It most
+        # certainly is not. Silly pylint.
+        #
+        # pylint: disable-msg=E1103
+        #
+        [obj, offset] = Gecko.Script.getBottomOfFile(self)
+        if obj and obj.getState().contains(pyatspi.STATE_EDITABLE):
+            offset += 1
+
+        return [obj, offset]
+
     def getDocumentFrame(self):
         """Returns the document frame that holds the content being shown.
         Overridden here because multiple open messages are not arranged
         in tabs like they are in Firefox."""
 
+        if self.inFindToolbar():
+            return Gecko.Script.getDocumentFrame(self)
+
         obj = orca_state.locusOfFocus
         while obj:
             role = obj.getRole()
@@ -585,3 +620,35 @@ class Script(Gecko.Script):
                 obj = obj.parent
 
         return None
+
+    def toggleFlatReviewMode(self, inputEvent=None):
+        """Toggles between flat review mode and focus tracking mode."""
+
+        # If we're leaving flat review dump the cache. See bug 568658.
+        #
+        if self.flatReviewContext:
+            pyatspi.clearCache()
+
+        return default.Script.toggleFlatReviewMode(self, inputEvent)
+
+    def isEditableMessage(self, obj):
+        """Returns True if this is a editable message."""
+
+        # For now, look specifically to see if this object is the
+        # document frame. If it's not, we cannot be sure how complex
+        # this message is and should let the Gecko code kick in.
+        #
+        if obj \
+           and obj.getRole() == pyatspi.ROLE_DOCUMENT_FRAME \
+           and obj.getState().contains(pyatspi.STATE_EDITABLE):
+            return True
+
+        return False
+
+    def useCaretNavigationModel(self, keyboardEvent):
+        """Returns True if we should do our own caret navigation."""
+
+        if self.isEditableMessage(orca_state.locusOfFocus):
+            return False
+
+        return Gecko.Script.useCaretNavigationModel(self, keyboardEvent)
